/*
 *	readconf.h
*/

#ifndef _CONFFILE_H_
#define _CONFFILE_H_

#include <sys/types.h>

#ifndef _CONFFILE_C_
#define EXTERN	extern
#else
#define EXTERN
#endif


/*
 * 	configuration dump macros
*/

#define CONF_S(a)	printf("%-28s %s\n", #a, a);
#define CONF_S2(a)	printf("  %-28s: %s\n", #a, a);
#define CONF_SS(a, b)	printf("%-28s %s\n", a, b);
#define CONF_SS2(a, b)	printf("  %-28s: %s\n", a, b);
#define CONF_D(a)	printf("%-28s %d\n", #a, a);
#define CONF_D2(a)	printf("  %-28s: %d\n", #a, a);
// signed int
#define CONF_II(a, b)	printf("%-28s %d\n", a, b);
// unsigned int
#define CONF_UU(a, b)	printf("%-28s %u\n", a, b);
// unsigned int octal
#define CONF_OO(a, b)	printf("%-28s 0%o\n", a, b);
// unsigned int hex
#define CONF_HH(a, b)	printf("%-28s 0x%x\n", a, b);
// double
#define CONF_FF(a, b)	printf("%-28s %lf\n", a, b);


/*
 * 	configuration flags
*/
#define CONF_FLAG_NONE		0x00

// UINT variables dump flags
#define CONF_FLAG_DUMP_MASK	0x0f
#define CONF_FLAG_HEX		0x01
#define CONF_FLAG_OCT		0x02

// values validation flags
#define CONF_FLAG_VALID_MASK	0xf0
#define CONF_FLAG_MINUS_1	0x10
#define CONF_FLAG_EMPTY		0x20


typedef enum { CONFIG_END = 0,
	CONFIG_BOOL, CONFIG_INT, CONFIG_PORT,
	CONFIG_UINT,
	CONFIG_STR, CONFIG_IP, 
	CONFIG_DOUBLE
} option_type;




typedef union {
	int i;
	uint u;
	char *s;
	double d;
} config_def_val;

typedef union {
	int *i;
	uint *u;
	char *s;
	double *d;
} config_val;

typedef union {
	int uint_flags;
	int path_flags;
	int file_flags;
} config_flags;

struct config_option {
	option_type type;
	char *name;
	void *val;
	size_t size;
	int flags;
//	config_flags type_flags;
	config_def_val def;
};



EXTERN int read_config(struct config_option config_options[], char *filename);
EXTERN int config_set_defaults(struct config_option config_options[]);
EXTERN void dump_config(struct config_option config_options[], char *ignore_prefix);


#undef EXTERN

#endif


